/**
 * Define api of apeman-api-db
 * @function api
 * @param {Apemandb} db - Instance of Apemandb class.
 * @param {Object} [options] - Optional settings
 * @param {boolean} [options.logging] - Enable logger
 * @returns {Object} - API configuration
 */
'use strict'

const co = require('co')
const aslogger = require('aslogger')

const describe = require('./describe')

/** @lends api */
function api (db, options = {}) {
  let logger = aslogger({ disabled: !options.logging })

  return {
    ping (pong = 'pong') {
      return co(function * () {
        logger.trace('pong:', pong)
        return pong
      })
    },

    setup (rootUsername, rootPassword) {
      const s = this
      return co(function * () {
        yield db.setup(rootUsername, rootPassword)
      })
    },

    sync (options = {}) {
      const s = this
      return co(function * () {
        yield db.sync(options)
      })
    },

    reset () {
      const s = this
      return co(function * () {
        yield db.reset()
      })
    },

    describe () {
      const s = this
      return co(function * () {
        return yield db.describe()
      })
    },

    close () {
      const s = this
      return co(function * () {
        return Promise.resolve(db.close())
      })
    },

    seed (options = {}) {
      const s = this
      return co(function * () {
        yield db.seed(options)
      })
    },

    drop (options = {}) {
      const s = this
      return co(function * () {
        yield db.drop(options)
      })
    },

    /**
     * A hook which runs before method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @returns {Promise}
     */
    $before (methodName, params) {
      const s = this
      let { state } = s
      return co(function * () {
        state.invokedAt = new Date()
        logger.info(`${methodName} started...`)
      })
    },
    /**
     * A hook which runs after method invoked
     * @param {string} methodName - Name of method to invoke
     * @param {Array} params - Prams for invocation
     * @param {?*} returns - Return values
     * @returns {Promise}
     */
    $after (methodName, params, returns) {
      const s = this
      let { state } = s
      return co(function * () {
        let took = new Date() - state.invokedAt
        logger.info(`...${methodName} done! (${took}ms)`)
      })
    },
    /**
     * Method specifications
     */
    $spec: describe()
  }
}

module.exports = api
